# Tamito KAJIYAMA <4 March 2003>
# $Id: equal_area_projection.py,v 1.1 2004/09/18 22:57:23 kajiyama Exp $

# Surface names for equal-area projection: 
#           +-------------+
#           |             |
#           |     Top     |
#           |             |
#  +--------+-------------+
#  |        |             |
#  |  Left  |    Right    |
#  | (Side) |   (Front)   |
#  +--------+-------------+

from Sketch import Rotation, Scale

import math

def transform_right(context):
    buffer = []
    for object in context.document.selection.GetObjects():
        p = object.LayoutPoint()
        new_object = object.Duplicate()
        new_object.Translate(-p)
        new_object.Transform(Rotation(45*math.pi/180))
        new_object.Transform(Scale(0.5774, 1.0))
        new_object.Transform(Rotation(-30*math.pi/180))
        new_object.Translate(p)
        buffer.append(new_object)
    context.document.RemoveSelected()
    for object in buffer:
        context.document.Insert(object)
    context.document.SelectObject(buffer)

def transform_left(context):
    buffer = []
    for object in context.document.selection.GetObjects():
        p = object.LayoutPoint()
        new_object = object.Duplicate()
        new_object.Translate(-p)
        new_object.Transform(Rotation(-45*math.pi/180))
        new_object.Transform(Scale(0.5774, 1.0))
        new_object.Transform(Rotation(30*math.pi/180))
        new_object.Translate(p)
        buffer.append(new_object)
    context.document.RemoveSelected()
    for object in buffer:
        context.document.Insert(object)
    context.document.SelectObject(buffer)

def transform_top(context):
    buffer = []
    for object in context.document.selection.GetObjects():
        p = object.LayoutPoint()
        new_object = object.Duplicate()
        new_object.Translate(-p)
        new_object.Transform(Rotation(-45*math.pi/180))
        new_object.Transform(Scale(0.5774, 1.0))
        new_object.Transform(Rotation(90*math.pi/180))
        new_object.Translate(p)
        buffer.append(new_object)
    context.document.RemoveSelected()
    for object in buffer:
        context.document.Insert(object)
    context.document.SelectObject(buffer)

import Sketch.Scripting

Sketch.Scripting.AddFunction("Equal-Area Projection: Right", "Right",
                             transform_right, menu="Equal-Area Projection",
                             script_type=Sketch.Scripting.AdvancedScript)
Sketch.Scripting.AddFunction("Equal-Area Projection: Left", "Left",
                             transform_left, menu="Equal-Area Projection",
                             script_type=Sketch.Scripting.AdvancedScript)
Sketch.Scripting.AddFunction("Equal-Area Projection: Top", "Top",
                             transform_top, menu="Equal-Area Projection",
                             script_type=Sketch.Scripting.AdvancedScript)
